/*
 * Copyright (C) The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package main.java.com.google.android.gms.samples.vision.face.photo;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.SparseArray;
import android.view.View;

import com.google.android.gms.vision.face.Face;
import com.google.android.gms.vision.face.Landmark;

import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

import anywheresoftware.b4a.BA;

/**
 * View which displays a bitmap containing a face along with overlay graphics that identify the
 * locations of detected facial landmarks.
 */
public class FaceView extends View {
    private Bitmap mBitmap;
    private SparseArray<Face> mFaces;
	Face myface;

    public FaceView(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    /**
     * Sets the bitmap background and the associated face detections.
     */
    public void setContent(Bitmap bitmap, SparseArray<Face> faces) {
        mBitmap = bitmap;
        mFaces = faces;
        invalidate();
    }

    /**
     * Draws the bitmap background and the associated face landmarks.
     */
    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        if ((mBitmap != null) && (mFaces != null)) {
            double scale = drawBitmap(canvas);
            drawFaceAnnotations(canvas, scale);
        }
    }

    /**
     * Draws the bitmap background, scaled to the device size.  Returns the scale for future use in
     * positioning the facial landmark graphics.
     */
    private double drawBitmap(Canvas canvas) {
        double viewWidth = canvas.getWidth();
        double viewHeight = canvas.getHeight();
        double imageWidth = mBitmap.getWidth();
        double imageHeight = mBitmap.getHeight();
        double scale = Math.min(viewWidth / imageWidth, viewHeight / imageHeight);

        Rect destBounds = new Rect(0, 0, (int)(imageWidth * scale), (int)(imageHeight * scale));
        canvas.drawBitmap(mBitmap, null, destBounds, null);
        return scale;
    }

    /**
     * Draws a small circle for each detected landmark, centered at the detected landmark position.
     * <p>
     *
     * Note that eye landmarks are defined to be the midpoint between the detected eye corner
     * positions, which tends to place the eye landmarks at the lower eyelid rather than at the
     * pupil position.
     */
	 
	
	private ArrayList<String> ar2 = new ArrayList<String>();	

    private void drawFaceAnnotations(Canvas canvas, double scale) {
        Paint paint = new Paint();
        paint.setColor(circleColor);
        paint.setStyle(Paint.Style.STROKE);
        paint.setStrokeWidth(circleWidth);

	    ArrayList<String> arl = new ArrayList<String>();	
        for (int i = 0; i < mFaces.size(); ++i) {
            Face face = mFaces.valueAt(i);
			myface = face;
            for (Landmark landmark : face.getLandmarks()) {
                int cx = (int) (landmark.getPosition().x * scale);
                int cy = (int) (landmark.getPosition().y * scale);
                canvas.drawCircle(cx, cy, circleDiameter, paint);
				arl.add("" + landmark.getType());
            }
		ar2 = arl;
		invokeFaceFound();	
        }
    }
	
	private int circleColor = Color.GREEN;
	public void setCircleColor(int circleColor) {
		this.circleColor = circleColor;
	}
	
	private int circleWidth = 5;
	public void setCircleWidth(int circleWidth) {
		this.circleWidth = circleWidth;
	}	
	
	private int circleDiameter = 10;
	public void setCircleDiameter(int circleDiameter) {
		this.circleDiameter = circleDiameter;
	}	

    
	
	public float getEulerY() {
	  return myface.getEulerY();
	}

	public float getEulerZ() {
	  return myface.getEulerZ();
	}
	

	public ArrayList<String> getLandmarks() {
       return ar2;		  
	}

	public float getIsLeftEyeOpenProbability() {
	  return myface.getIsLeftEyeOpenProbability();
	}

	public float getIsRightEyeOpenProbability() {
	  return myface.getIsRightEyeOpenProbability();
	}

	public float getIsSmilingProbability() {
	  return myface.getIsSmilingProbability();
	}

	public int getId() {
	  return myface.getId();
	}


	
	
    Callback callback;
    public interface  Callback{
        void face_found();
    }

    private void invokeFaceFound() {
        if(callback!=null){
            callback.face_found();
        }
    }


    public void setCallback(Callback callback){
        this.callback = callback;
    }		



 
	
	
}
